@@ -1,12 +1,12 @@
 use std::env;
-use std::hash::{Hash, Hasher};
 
 use chrono::Utc;
 
 use atuin_common::utils::uuid_v4;
+use serde::{Deserialize, Serialize};
 
 // Any new fields MUST be Optional<>!
-#[derive(Debug, Clone, Serialize, Deserialize, Ord, PartialOrd, sqlx::FromRow)]
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, sqlx::FromRow)]
 pub struct History {
     pub id: String,
     pub timestamp: chrono::DateTime<Utc>,
@@ -46,21 +46,3 @@ impl History {
         }
     }
 }
-
-impl PartialEq for History {
-    // for the sakes of listing unique history only, we do not care about
-    // anything else
-    // obviously this does not refer to the *same* item of history, but when
-    // we only render the command, it looks the same
-    fn eq(&self, other: &Self) -> bool {
-        self.command == other.command
-    }
-}
-
-impl Eq for History {}
-
-impl Hash for History {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.command.hash(state);
-    }
-}
