@@ -3,13 +3,15 @@
 #[macro_use]
 extern crate log;
 
-#[macro_use]
-extern crate serde_derive;
-
+#[cfg(feature = "sync")]
 pub mod api_client;
-pub mod database;
+#[cfg(feature = "sync")]
 pub mod encryption;
+#[cfg(feature = "sync")]
+pub mod sync;
+
+pub mod database;
 pub mod history;
 pub mod import;
+pub mod ordering;
 pub mod settings;
-pub mod sync;
