@@ -1,4 +1,5 @@
 use chrono::Utc;
+use serde::{Deserialize, Serialize};
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct UserResponse {
@@ -52,19 +53,3 @@ pub struct SyncHistoryRequest {
 pub struct SyncHistoryResponse {
     pub history: Vec<String>,
 }
-
-#[derive(Debug, Serialize, Deserialize)]
-pub struct ErrorResponse {
-    pub reason: String,
-}
-
-impl ErrorResponse {
-    pub fn reply(reason: &str, status: warp::http::StatusCode) -> impl warp::Reply {
-        warp::reply::with_status(
-            warp::reply::json(&ErrorResponse {
-                reason: String::from(reason),
-            }),
-            status,
-        )
-    }
-}
